#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

uniform int levels;
//float levels = 5;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec3 color_world;
	vec2 uv_world;
} vert;

uniform vec3 camera_world;

// Texture
uniform bool hasTexture;
uniform sampler2D textureSampler;

// SpecMap
uniform bool hasSpecMap;
uniform sampler2D specMapSampler;

// Lights

#define MAX_LIGHTS 10

uniform int dl_count;

uniform vec3 dl_direction[MAX_LIGHTS];
uniform vec3 dl_color[MAX_LIGHTS];

uniform int pl_count;

uniform vec3 pl_position[MAX_LIGHTS];
uniform vec3 pl_color[MAX_LIGHTS];
uniform vec3 pl_attenuation[MAX_LIGHTS];

uniform int sl_count;

uniform vec3 sl_position[MAX_LIGHTS];
uniform vec3 sl_direction[MAX_LIGHTS];
uniform vec3 sl_color[MAX_LIGHTS];

uniform float sl_inner_angle[MAX_LIGHTS];
uniform float sl_outer_angle[MAX_LIGHTS];
uniform vec3 sl_attenuation[MAX_LIGHTS];

// Color & Material

vec3 ambientColor = vec3(1,1,1);
uniform vec3 diffuseColor;

uniform float ambientReflection;
uniform float diffuseReflection;
uniform float specularReflection;
uniform float shininess;

out vec4 color_out;

void main() {	
	vec3 normal_world = normalize(vert.normal_world);
	vec3 to_viewer_vec = normalize(camera_world-vert.position_world);
	vec3 objectColor = diffuseColor+vert.color_world;
	vec3 textureColor = vec3(0,0,0);
	
	float diff = 0;
	float spec = 0;
	
	if(hasTexture) {
		//vec3(vert.uv_world.x,0,vert.uv_world.y);//
		textureColor = texture(textureSampler, vert.uv_world).rgb;
		objectColor = textureColor+objectColor;
	}
	
	float specMapIntensity = specularReflection;
	
	if(hasSpecMap) {
		specMapIntensity = texture(specMapSampler, vert.uv_world).x;
	}
	
	vec3 color;
	color = vec3(0,0,0);
	
	// ambient
	color += objectColor*ambientReflection*ambientColor;
	
	//Directional Light
	for(int i=0;i<MAX_LIGHTS;i++) {
		if(i<dl_count) {
	
			vec3 to_dlight_vec = normalize(-dl_direction[i]);
		
		/*	// diffuse
			diff = (max(dot(normal_world,to_dlight_vec), 0));
			color += objectColor*dl_color[i]*diffuseReflection* diff;
			
			// specular
			spec = pow(max(dot(to_viewer_vec,reflect(-to_dlight_vec,normal_world)),0), shininess);
			color += specMapIntensity*dl_color[i]* spec;*/

			diff = (max(dot(normal_world,to_dlight_vec), 0));
			spec = pow(max(dot(to_viewer_vec,reflect(-to_dlight_vec,normal_world)),0), shininess);

		
			float brightness = diff;
			brightness = ceil(brightness*levels)/levels;
			
			
			// diffuse
			color += objectColor*dl_color[i]*diffuseReflection* brightness;
			
			
			brightness = spec;
			brightness = ceil(brightness*levels)/levels;
			
			// specular
			color += specMapIntensity*dl_color[i]*brightness;	
		}
	}

    //Point Light
	for(int i=0;i<MAX_LIGHTS;i++) {
	
		if(i<pl_count) {
	
			vec3 to_plight_vec = normalize(pl_position[i]-vert.position_world);
	
			float pl_distance = length(pl_position[i]-vert.position_world);
			float pl_attenuationCoefficient = 1.0f;
			pl_attenuationCoefficient /= (pl_attenuation[i].x+pl_attenuation[i].y*pl_distance+pl_attenuation[i].z*pl_distance*pl_distance);
		
			/*// diffuse
			diff = (max(dot(normal_world,to_plight_vec), 0));
			color += objectColor*pl_attenuationCoefficient*pl_color[i]*diffuseReflection*diff;
			// specular
			spec = pow(max(dot(to_viewer_vec,reflect(-to_plight_vec,normal_world)),0), shininess);
			color += specMapIntensity*pl_attenuationCoefficient*pl_color[i]*spec;*/
			
			diff = (max(dot(normal_world,to_plight_vec), 0));
			spec = pow(max(dot(to_viewer_vec,reflect(-to_plight_vec,normal_world)),0), shininess);
			
			float brightness = diff+spec;
			brightness = ceil(brightness*levels)/levels;
			
			// diffuse
			color += objectColor*pl_attenuationCoefficient*pl_color[i]*diffuseReflection*brightness;
			// specular
			color += specMapIntensity*pl_attenuationCoefficient*pl_color[i]*brightness;
		}
	}
	
	//Spot Light
	for(int i=0;i<MAX_LIGHTS;i++) {
	
		if(i<sl_count) {
	
			vec3 spot_vec = normalize(-sl_direction[i]);
			vec3 to_slight_vec = normalize(sl_position[i]-vert.position_world);//normalize(sl_position[i]-vert.position_world);
	
	
			if(max(dot(spot_vec,to_slight_vec), 0) > cos(sl_outer_angle[i])) {
	
				float sl_distance = length(sl_position[i]-vert.position_world);
				float sl_attenuationCoefficient = 1.0f;
				sl_attenuationCoefficient /= (sl_attenuation[i].x+sl_attenuation[i].y*sl_distance+sl_attenuation[i].z*sl_distance*sl_distance);
		
				// interpolate
				float outer_factor = smoothstep(cos(sl_outer_angle[i]), cos(sl_inner_angle[i]), max(dot(spot_vec,to_slight_vec), 0));
				float intensity = max(dot(normal_world,to_slight_vec), 0);
				
				if(max(dot(spot_vec,to_slight_vec), 0) > cos(sl_inner_angle[i])) {
					outer_factor = 1;
				}
				
				if(intensity > 0.0) {
				
					// diffuse
					color += intensity*objectColor*sl_attenuationCoefficient*sl_color[i]*diffuseReflection*(outer_factor);
					// specular
					spec = pow(max(dot(to_viewer_vec,reflect(-to_slight_vec,normal_world)),0), shininess);
					color += specMapIntensity*sl_attenuationCoefficient*sl_color[i]*(outer_factor);
				}
			}
		}
	}
	
	color_out = vec4(color, 1);
}