#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec3 color_world;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;

uniform vec3 camera_world;

// Lights

#define MAX_LIGHTS 10

uniform int dl_count;

uniform vec3 dl_direction[MAX_LIGHTS];
uniform vec3 dl_color[MAX_LIGHTS];

uniform int pl_count;

uniform vec3 pl_position[MAX_LIGHTS];
uniform vec3 pl_color[MAX_LIGHTS];
uniform vec3 pl_attenuation[MAX_LIGHTS];

uniform int sl_count;

uniform vec3 sl_position[MAX_LIGHTS];
uniform vec3 sl_direction[MAX_LIGHTS];
uniform vec3 sl_color[MAX_LIGHTS];

uniform float sl_inner_angle[MAX_LIGHTS];
uniform float sl_outer_angle[MAX_LIGHTS];
uniform vec3 sl_attenuation[MAX_LIGHTS];

// Color & Material

vec3 ambientColor = vec3(1,1,1);
uniform vec3 diffuseColor;

uniform float ambientReflection;
uniform float diffuseReflection;
uniform float specularReflection;
uniform float shininess;

void main() {
	gl_Position = viewProjMatrix * modelMatrix * vec4(position, 1);
	vert.position_world = vec3(modelMatrix * vec4(position, 1));

	vert.normal_world = vec3(transpose(inverse(modelMatrix)) * vec4(normal, 1)); // normals fix
	vert.normal_world = normalize(vert.normal_world);
	
	vec3 to_viewer_vec = normalize(camera_world-vert.position_world);
	
	vert.color_world = vec3(0,0,0);
	// ambient
	vert.color_world += diffuseColor*ambientReflection*ambientColor;
	
	
	for(int i=0;i<MAX_LIGHTS;i++) {
		if(i<dl_count) {
	
			vec3 to_dlight_vec = normalize(-dl_direction[i]);
		
			// diffuse
			vert.color_world += diffuseColor*dl_color[i]*diffuseReflection*(max(dot(vert.normal_world,to_dlight_vec), 0));
			// specular
			vert.color_world += dl_color[i]*specularReflection*pow(max(dot(to_viewer_vec,reflect(-to_dlight_vec,vert.normal_world)),0), shininess);
		}
	}


	for(int i=0;i<MAX_LIGHTS;i++) {
	
		if(i<pl_count) {
	
			vec3 to_plight_vec = normalize(pl_position[i]-vert.position_world);
	
			float pl_distance = length(pl_position[i]-vert.position_world);
			float pl_attenuationCoefficient = 1.0f;
			pl_attenuationCoefficient /= (pl_attenuation[i].x+pl_attenuation[i].y*pl_distance+pl_attenuation[i].z*pl_distance*pl_distance);
		
			// diffuse
			vert.color_world += diffuseColor*pl_attenuationCoefficient*pl_color[i]*diffuseReflection*(max(dot(vert.normal_world,to_plight_vec), 0));
			// specular
			vert.color_world += pl_attenuationCoefficient*pl_color[i]*specularReflection*pow(max(dot(to_viewer_vec,reflect(-to_plight_vec,vert.normal_world)),0), shininess);
		}
	}
	
	
	
	for(int i=0;i<MAX_LIGHTS;i++) {
	
		if(i<sl_count) {
	
			vec3 spot_vec = normalize(-sl_direction[i]);
			vec3 to_slight_vec = normalize(sl_position[i]-vert.position_world);//normalize(sl_position[i]-vert.position_world);
	
	
			if(max(dot(spot_vec,to_slight_vec), 0) > cos(sl_outer_angle[i])) {
	
				float sl_distance = length(sl_position[i]-vert.position_world);
				float sl_attenuationCoefficient = 1.0f;
				sl_attenuationCoefficient /= (sl_attenuation[i].x+sl_attenuation[i].y*sl_distance+sl_attenuation[i].z*sl_distance*sl_distance);
		
				// interpolate
				float outer_factor = smoothstep(cos(sl_outer_angle[i]), cos(sl_inner_angle[i]), max(dot(spot_vec,to_slight_vec), 0));
				float intensity = max(dot(vert.normal_world,to_slight_vec), 0);
				
				if(max(dot(spot_vec,to_slight_vec), 0) > cos(sl_inner_angle[i])) {
					outer_factor = 1;
				}
				
				if(intensity >= 0.0) {
					// diffuse
					vert.color_world += diffuseColor*sl_attenuationCoefficient*sl_color[i]*diffuseReflection*(outer_factor);
					// specular
					vert.color_world += sl_attenuationCoefficient*sl_color[i]*specularReflection*pow(max(dot(to_viewer_vec,reflect(-to_slight_vec,vert.normal_world)),0), shininess);
				}
			}
		}
	}
	
}
	
	
	
	